# This code  performs a non-linear fit for the short-term transient response as described in
# Yang et al. (2018) "Variability, Timescales, and Non-linearity in Climate Responses to Black Carbon Emissions"
#
# This routine reads in annual time series of the surface temperature response averaged over either mid-latitudes or Arctic for a number of ensembles
#  
# The transient response is assumed to take the form of a single exponential.
# The primary result of this calcuation is the exponential timescale for the initial transient.
#
# Set working directory to the location of the two data files supplied with code

# See: https://github.com/rmcelreath/rethinking
library('rethinking')

# Pass in time series and long-term response magnitude and use Hamiltonian Monte Carlo estimation
# to estimate transient timescale 
fit_with_map2stan <- function( data_in, lower_tau_init = 1, upper_tau_init = 10 ) {
  
  # Extract average short-term perturbation time series
  Timeseries_Ave <- Extract_Short_Term_Ave( data_in )
  
  # Calculate stats
  # Long-term average with its SD is used as prior
  long_term_statistics <- Calc_Long_Term_Stats( data_in )
  long_term_average <- long_term_statistics$lt_ave
  stdev <- long_term_statistics$sd
  
  # Note "-1/tau * t" formulation is required for map2stan parser
  model_specification <- list(
    quote(y ~ dnorm(mu, sigma)),
    substitute(mu <- long_term_ave * (1 - exp(-1/tau * t) ) ),
    substitute( tau ~ dunif( lower_tau, upper_tau ), 
               list(lower_tau = lower_tau_init, upper_tau = upper_tau_init)),
    quote(sigma ~ dunif( 0, 5 )),
    substitute( long_term_ave ~ dnorm( lta, std ),
                list( lta = long_term_average, std = stdev ) )
  )
  
  map_results <- rethinking::map2stan(
    model_specification,
    data=Timeseries_Ave,
    control=list(adapt_delta=0.9)
  )
  
  pairs( map_results@stanfit )
  
  return( map_results )
}

# Select portion of dataset with specified perturbation
Select_Data <- function( data_in, data_name ) {
  # Select specified sub-set
  Selected_Names <- grep( paste0(data_name,"*"), names( data_in ), value=TRUE )
  Selected_data <- as.data.frame( data_in )
  Selected_data <- Selected_data[ , Selected_Names ]
  
  return( Selected_data )
}

# Extract average of short-term ensemble results
Extract_Short_Term_Ave <- function( data_in, data_name, short_term_end = 30 ) {

  # Create time series of average over points up to short_term_end
  Number_Ens <- length( names( data_in ) ) - 1
  data_Average <- rowSums( data_in[1:short_term_end, 1:5 ] ) / Number_Ens
  data_Average <- as.data.frame( data_Average )
  data_Average$t <- 1:nrow( data_Average )
  names( data_Average ) <- c("y","t")
  return( data_Average )
}

# Calculate SD and mean of all long-term data points
Calc_Long_Term_Stats <- function( data_in, long_term_ave_start = 11 ) {

  # Select data points to be used for long-term mean
  long_term_data <- data_in[ long_term_ave_start:nrow( data_in ), ]
  long_term_data <- unlist( long_term_data )
  
  long_term_average <- mean( na.omit( long_term_data ) )
  standard_deivation <- sd( na.omit( long_term_data ) )
  
  resultsList <- list( "lt_ave" = long_term_average, "sd" = standard_deivation )

  return( resultsList )
}

# Do a linear fit for last 90 years of first ensemble member
# This checks for any linear trend after the initial transient period
# Results from the 100 year run will be the first time series in each set of data
Check_for_Linear_Slope <- function( data_in, long_term_ave_start = 11 ) {
  
  # Select data points to be used for long-term mean
  long_term_series <- data_in[ long_term_ave_start:nrow( data_in ), 1 ]
  long_term_series <- as.data.frame( long_term_series )
  
  # Create time series of average over points up to short_term_end
  long_term_series$t <- 1:nrow( long_term_series )
  names( long_term_series ) <- c("response","t")
  
  linearModel <- lm( response ~ t, long_term_series )
 # Write out summary of linear fit 
  print("Linear trend from 10-100 years:")
  
  print( summary( linearModel ) )
}

# Call fitting routines and print out summary results
Fit_The_Curves <- function ( selected_data, perturbation_name, output_name ) {
  
	print("Performing STAN fit")
	stan_results = fit_with_map2stan( selected_data )

	# Print statistical summaries
	long_term_statistics <- Calc_Long_Term_Stats( selected_data )
	# The long-term average is the average of points from all available ensemble data after the first 10 years.
	long_term_average <- long_term_statistics$lt_ave
	stddev <- long_term_statistics$sd
	print(paste0(output_name, " - ", perturbation_name," BC perturbation - long-term ave: ",long_term_average, " (±", stddev, ")"))
 
    print("stan Hamiltonian Monte Carlo fit results:")
    print( precis( stan_results, prob=0.90 ) )
    # Save detailed graphical summary for this fit
    dev.copy(png,paste0(output_name, " - ", perturbation_name," BC - detailed pairs",".png") )
    dev.off()
    pairs( stan_results )
    dev.copy(png,paste0(output_name, " - ", perturbation_name," BC - pairs",".png") )
    dev.off()
 
}

#************************************************************************************
# Main Calcuation 
#************************************************************************************

# For each set of ensemble members, 
# Read in mid-latitude temperature data
temperature_data <- read.csv("Mid-LatTemp.csv", skip = 3)

# Mid-latitude temperature change resulting from the 7X mid-latitudes emissions perturbation
output_name <- "Mid-Latitude Temperature"
perturbation_name <- "MID7X"
selected_results <- Select_Data( temperature_data, perturbation_name )
Fit_The_Curves ( selected_results, perturbation_name, output_name ) 
Check_for_Linear_Slope( selected_results )

# Mid-latitude temperature change resulting from the 150X Arctic emissions perturbation
perturbation_name <- "ARC150X"
selected_results <- Select_Data( temperature_data, perturbation_name )
Fit_The_Curves ( selected_results, perturbation_name, output_name ) 
Check_for_Linear_Slope( selected_results )

# Read in Arctic temperature data
temperature_data <- read.csv("Arctic-Temp.csv", skip = 3)

# Arctic temperature change resulting from the 7X mid-latitudes emissions perturbation
output_name <- "Arctic Temperature"
perturbation_name <- "MID7X"
selected_results <- Select_Data( temperature_data, perturbation_name )
Fit_The_Curves ( selected_results, perturbation_name, output_name ) 
Check_for_Linear_Slope( selected_results )

# Arctic temperature change resulting from the 150X Arctic emissions perturbation
output_name <- "Arctic Temperature"
perturbation_name <- "ARC150X"
selected_results <- Select_Data( temperature_data, perturbation_name )
Fit_The_Curves ( selected_results, perturbation_name, output_name ) 
Check_for_Linear_Slope( selected_results )

